function [B,Sigma,T,Lambda] = dfa(Q,H,R)

% Inputs: Q, H and R
% Given the State Space model 
%
%   x(t+1) = x(t) + w(t),       var{w(t)} = Q
%   y(t)   = H*x(t) + v(t),     var{v(t)} = R
%
% dimension of x(t) is m, and dimension of y(t) = n, being n>m
% C is n x m , Q is m x m and R is n x n
%
% (1) Outputs: T and Lambda
%
% T is a non singular matrix that simultaneously
% diagonalizes HQH' an R, so that 
%             THQH'T' = Lambda
%             TRT' = Identity
%  and z(t) = T y(t), f(t) = THx(t) verify
%   
%   f(t+1)  = f(t) + a(t);      var(a(t)) = Lambda
%   z(t)    = f(t) + e(t);      var{e(t)} = Identity
%
% Lambda is a diagonal matrix with n-m values equal to zero,
% is given as a vector with m positive values
%
% (2)Outputs: B, Sigma 
%
% (yhat(t) is the predicted value for y(t))
% The exponentail smoothing recursion
%
%     yhat(t+1) = yhat(t) + B(y(t)-yhat(t)), var(y(t)-yhat(t))=Sigma
%
 


     m = max(size(Q));
     n = max(size(R));
     
% Computation of T and Lambda
    L = chol(R)';
    [V1,D1] = svd(Q);
    M = L\H*V1*sqrt(D1);
    [V,G] = svd(M);
    G = G(1:m,1:m);
    T = V'/L;
    Lambda = (diag(G)).^2;
    %
%   Computation of D, P, B, Omega and Sigma
    D = (-Lambda+sqrt(Lambda.^2+4*Lambda))/2; 
    Omega = D + Lambda;
%   
    Omegac = [Omega' zeros(1,n-m)]';
    Dc = [D' zeros(1,n-m)]';
    P = T\diag(Omegac)/(T');
    B = T\diag(Dc)*T;
    Sigma = P + R;
    